/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app.h>
#include <dlog.h>
#include <system_settings.h>
#include <efl_extension.h>

#include "$(appName).h"
#include "view.h"
#include "data.h"

static Eina_Bool _naviframe_pop_cb(void *data, Elm_Object_Item *it);
static void _create_detailed_view(void *data, Evas_Object *obj, void *event_info);

/**
 * @brief Hook to take necessary actions before main event loop starts.
 * Initialize UI resources and application's data.
 * If this function returns true, the main loop of application starts.
 * If this function returns false, the application is terminated.
 */
static bool app_create(void *user_data)
{
	Evas_Object *nf = NULL;
	Evas_Object *genlist = NULL;
	int count = 0;
	int i = 0;

	view_create();

	nf = view_get_naviframe();
	genlist = view_create_circle_genlist(nf);

	/**
	 * Append a genlist's item as a title that has group index.
	 */
	view_append_item_to_genlist(genlist, "title_with_groupindex", NULL, NULL, NULL);


	/**
	 * Get count of specific item.
	 * In this case, it can get count of favourites items,
	 */
	count = data_get_contact_count(CONTACT_TYPE_FAVOURITES);
	if (count > 0) {
		/**
		 * Append a genlist's item as a group index if there are members in group.
		 */
		view_append_item_to_genlist(genlist, "groupindex", (void *) CONTACT_TYPE_FAVOURITES, NULL, NULL);

		for (i = 0; i < count; i++) {
			contact_info_s *contact_info = malloc(sizeof(contact_info_s));

			contact_info->type = CONTACT_TYPE_FAVOURITES;
			contact_info->index = i;
			contact_info->name = data_get_contact_name(CONTACT_TYPE_FAVOURITES, i);
			contact_info->number = data_get_contact_number(CONTACT_TYPE_FAVOURITES, i);
			contact_info->img_path = data_get_contact_image(CONTACT_TYPE_FAVOURITES, i);

			/**
			 * Append items to genlist.
			 */
			view_append_item_to_genlist(genlist, "1text.1icon", contact_info,
					_create_detailed_view, contact_info);
		}
	}

	count = data_get_contact_count(CONTACT_TYPE_MY_PROFILE);
	if (count > 0) {
		view_append_item_to_genlist(genlist, "groupindex", (void *) CONTACT_TYPE_MY_PROFILE, NULL, NULL);

		for (i = 0; i < count; i++) {
			contact_info_s *contact_info = malloc(sizeof(contact_info_s));

			contact_info->type = CONTACT_TYPE_MY_PROFILE;
			contact_info->index = i;
			contact_info->name = data_get_contact_name(CONTACT_TYPE_MY_PROFILE, i);
			contact_info->number = data_get_contact_number(CONTACT_TYPE_MY_PROFILE, i);
			contact_info->img_path = data_get_contact_image(CONTACT_TYPE_MY_PROFILE, i);

			view_append_item_to_genlist(genlist, "1text.1icon", contact_info,
					_create_detailed_view, contact_info);
		}
	}

	count = data_get_contact_count(CONTACT_TYPE_CONTACTS);
	if (count > 0) {
		view_append_item_to_genlist(genlist, "groupindex", (void *) CONTACT_TYPE_CONTACTS, NULL, NULL);

		for (i = 0; i < count; i++) {
			contact_info_s *contact_info = malloc(sizeof(contact_info_s));

			contact_info->type = CONTACT_TYPE_CONTACTS;
			contact_info->index = i;
			contact_info->name = data_get_contact_name(CONTACT_TYPE_CONTACTS, i);
			contact_info->number = data_get_contact_number(CONTACT_TYPE_CONTACTS, i);
			contact_info->img_path = data_get_contact_image(CONTACT_TYPE_CONTACTS, i);

			view_append_item_to_genlist(genlist, "1text.1icon", contact_info,
					_create_detailed_view, contact_info);
		}
	}

	/**
	 * Append a genlist's item as a padding item.
	 * Padding item makes genlist's items is located at the middle of the screen.
	 */
	view_append_item_to_genlist(genlist, "padding", NULL, NULL, NULL);

	/**
	 * Push the genlist at naviframe.
	 */
	view_push_item_to_naviframe(nf, genlist, _naviframe_pop_cb, NULL);

	view_set_genlist(genlist);

	return true;
}

/**
 * @brief This callback function is called when another application
 * sends the launch request to the application.
 */
static void app_control(app_control_h app_control, void *user_data)
{
	/* Handle the launch request. */
}

/**
 * @brief This callback function is called each time
 * the application is completely obscured by another application
 * and becomes invisible to the user.
 */
static void app_pause(void *user_data)
{
	/* Take necessary actions when application becomes invisible. */
}

/**
 * @brief This callback function is called each time
 * the application becomes visible to the user.
 */
static void app_resume(void *user_data)
{
	/* Take necessary actions when application becomes visible. */
}

/**
 * @brief This callback function is called once after the main loop of the application exits.
 */
static void app_terminate(void *user_data)
{
	/* Release all resources. */
	Evas_Object *genlist = view_get_genlist();
	int gl_it_count = 0;
	int i = 0;

	gl_it_count = elm_genlist_items_count(genlist);

	for (i = 0; i < gl_it_count; i++) {
		Elm_Object_Item *it = NULL;

		it = elm_genlist_nth_item_get(genlist, i);
		if (elm_genlist_item_type_get(it) == ELM_GENLIST_ITEM_NONE) {
			contact_info_s *contact_info = NULL;

			contact_info = (contact_info_s *) elm_object_item_data_get(it);
			free(contact_info);
		}
	}

	view_destroy();
}

/**
 * @brief This function will be called when the language is changed.
 */
static void ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;

	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);

	if (locale != NULL) {
		elm_language_set(locale);
		free(locale);
	}
	return;
}

/**
 * @brief Main function of the application.
 */
int main(int argc, char *argv[])
{
	int ret;

	ui_app_lifecycle_callback_s event_callback = {0, };
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	/**
	 * If you want to handling more events,
	 * Please check the application lifecycle guide.
	 */
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, NULL);

	ret = ui_app_main(argc, argv, &event_callback, NULL);
	if (ret != APP_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "ui_app_main() failed. err = %d", ret);

	return ret;
}

/**
 * @note
 * Below functions are static.
 */

/**
 * @brief This function will be operated when the first item of the naviframe is going to popped.
 * @param[in] data The data is user specific data
 * @param[in] it The item of naviframe object
 */
static Eina_Bool _naviframe_pop_cb(void *data, Elm_Object_Item *it)
{
	ui_app_exit();

	return EINA_FALSE;
}

/**
 * @brief The convenience function that is called when the item is selected.
 * @param[in] data The data passed from 'elm_genlist_item_append' as seventh parameter
 * @param[in] obj The genlist object
 * @param[in] event_info A pointer to data which is totally dependent on the smart object's implementation and semantic for the given event
 */
static void _create_detailed_view(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *nf = NULL;
	Evas_Object *detail_view_layout = NULL;
	contact_info_s *contact_info = (contact_info_s *) data;
	char edj_path[PATH_MAX] = { 0, };

	nf = view_get_naviframe();

	data_get_resource_path(EDJ_FILE, edj_path, (int)PATH_MAX);

	detail_view_layout = view_create_layout(nf, edj_path, "detail-view", NULL, NULL);

	view_set_text(detail_view_layout, "name.txt", contact_info->name);
	view_set_text(detail_view_layout, "number.txt", contact_info->number);

	if (contact_info->img_path != NULL) {
		view_set_image(detail_view_layout, "bg.swallow", contact_info->img_path);
	}

	view_push_item_to_naviframe(nf, detail_view_layout, NULL, NULL);
}

/* End of file */
