/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <tizen.h>
#include <dlog.h>
#include <efl_extension.h>
#include <Elementary.h>
#include <app.h>

#include "$(appName).h"
#include "view.h"
#include "data.h"

static struct view_info {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *layout;
	Evas_Object *nf;
	Evas_Object *genlist;
	Eext_Circle_Surface *circle_surface;
} s_info = {
	.win = NULL,
	.conform = NULL,
	.layout = NULL,
	.nf = NULL,
	.genlist = NULL,
	.circle_surface = NULL,
};

#define IMAGE_CONTACT_BG "images/wc_contact_bg.png"

static Elm_Genlist_Item_Class *_set_genlist_item_class(const char *style);
static void _win_delete_request_cb(void *data, Evas_Object *obj, void *event_info);
static void _gl_selected_cb(void *data, Evas_Object *obj, void *event_info);
static Evas_Object* _get_gl_icon(void *data, Evas_Object *obj, const char *part);
static Evas_Object *_create_icon_layout(Evas_Object *parent, const char *name, const char *img_path, void *user_data);

/**
 * @brief Gets naviframe object.
 */
Evas_Object *view_get_naviframe(void)
{
	return s_info.nf;
}

/**
 * @brief Gets genlist object.
 */
Evas_Object *view_get_genlist(void)
{
	return s_info.genlist;
}

/**
 * @brief Sets genlist object.
 * @param[in] genlist The genlist object will be stored in view_info
 */
void view_set_genlist(Evas_Object *genlist)
{
	s_info.genlist = genlist;
}

/**
 * @brief Creates essential objects: window, conformant and layout.
 */
Eina_Bool view_create(void)
{
	/**
	 * Create window object.
	 */
	s_info.win = view_create_win(PACKAGE);
	if (s_info.win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return EINA_FALSE;
	}

	/**
	 * Create conformant object.
	 */
	s_info.conform = view_create_conformant_without_indicator(s_info.win);
	if (s_info.conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		evas_object_del(s_info.win);
		return EINA_FALSE;
	}

	/**
	 * Create eext circle surface.
	 */
	s_info.circle_surface = eext_circle_surface_conformant_add(s_info.conform);

	s_info.layout = view_create_layout_for_conformant(s_info.conform, NULL, NULL, NULL, NULL);

	s_info.nf = view_create_naviframe(s_info.layout);
	if (s_info.nf == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a naviframe.");
		evas_object_del(s_info.win);
		return EINA_FALSE;
	}

	/**
	 * Show window after main view is set up.
	 */
	evas_object_show(s_info.win);

	return EINA_TRUE;
}

/**
 * @brief Creates a basic window named package.
 * @param[in] pkg_name Name of the window
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;

	/**
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate window.
	 */

	win = elm_win_util_standard_add(pkg_name, pkg_name);
	elm_win_conformant_set(win, EINA_TRUE);
	elm_win_autodel_set(win, EINA_TRUE);

	if (elm_win_wm_rotation_supported_get(win)) {
		int rots[4] = { 0, 90, 180, 270 };
		elm_win_wm_rotation_available_rotations_set(win, (const int *)(&rots), 4);
	}

	evas_object_smart_callback_add(win, "delete,request", _win_delete_request_cb, NULL);

	return win;
}

/**
 * @brief Creates a conformant without indicator for wearable app.
 * @param[in] win The object to which you want to set this conformant
 */
Evas_Object *view_create_conformant_without_indicator(Evas_Object *win)
{
	Evas_Object *conform = NULL;

	/* *
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);
	evas_object_show(conform);

	return conform;
}

/**
 * @brief Creates a layout to target parent object with EDJ file.
 * @param[in] parent The object to which you want to add this layout
 * @param[in] file_path File path of EDJ file will be used
 * @param[in] group_name The name of group in EDJ you want to set to
 * @param[in] cb_function The function will be called when back event is detected
 * @param[in] user_data The user data to be passed to the callback functions
 */
Evas_Object *view_create_layout(Evas_Object *parent, const char *file_path, const char *group_name, Eext_Event_Cb cb_function, void *user_data)
{
	Evas_Object *layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	layout = elm_layout_add(parent);
	elm_layout_file_set(layout, file_path, group_name);

	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	if (cb_function)
		eext_object_event_callback_add(layout, EEXT_CALLBACK_BACK, cb_function, user_data);

	evas_object_show(layout);

	return layout;
}

/**
 * @brief Creates a layout with theme.
 * @param[in] parent The object to which you want to add this layout
 * @param[in] class_name The class of the group
 * @param[in] group The group name in EDJ file you want to set to layout
 * @param[in] style The style to use
 */
Evas_Object *view_create_layout_by_theme(Evas_Object *parent, const char *class_name, const char *group, const char *style)
{
	Evas_Object *layout = NULL;

	/**
	 * Layout
	 * Create and initialize elm_layout.
	 * view_create_layout_by_theme() is used to create layout by using pre-made edje file.
	 */

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	layout = elm_layout_add(parent);
	elm_layout_theme_set(layout, class_name, group, style);
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	evas_object_show(layout);

	return layout;
}

/**
 * @brief Creates and sets a layout to conformant.
 * @param[in] parent The conformant object
 * @param[in] file_name The file path of EDJ will be used
 * @param[in] group_name The group name in EDJ you want to set to layout
 * @param[in] cb_function The function will be called when the back event is detected
 * @param[in] user_data The user data to be passed to the callback functions
 */
Evas_Object *view_create_layout_for_conformant(Evas_Object *parent, const char *file_name, const char *group_name, Eext_Event_Cb cb_function, void *user_data)
{
	Evas_Object *layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	if (file_name == NULL) {
		layout = view_create_layout_by_theme(parent, "layout", "application", "default");
	} else {
		layout = view_create_layout(parent, file_name, group_name, cb_function, user_data);
	}

	if (layout == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "layout is NULL.");
		return NULL;
	}

	elm_object_content_set(parent, layout);

	return layout;
}

/**
 * @brief Destroys window and frees its resources.
 */
void view_destroy(void)
{
	if (s_info.win == NULL)
		return;

	evas_object_del(s_info.win);
}

/**
 * @brief Sets image to given part.
 * @param[in] parent The object has part to which you want to set this image
 * @param[in] part_name The part name to which you want to set this image
 * @param[in] image_path The path of the image file
 */
void view_set_image(Evas_Object *parent, const char *part_name, const char *image_path)
{
	Evas_Object *image = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	image = elm_object_part_content_get(parent, part_name);
	if (image == NULL) {
		image = elm_image_add(parent);
		if (image == NULL) {
			dlog_print(DLOG_ERROR, LOG_TAG, "failed to create an image object.");
			return;
		}

		elm_object_part_content_set(parent, part_name, image);
	}

	if (EINA_FALSE == elm_image_file_set(image, image_path, NULL)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to set image.");
		return;
	}

	evas_object_show(image);

	return;
}

/**
 * @brief Sets text to the part.
 * @param[in] parent The object has part to which you want to set text
 * @param[in] part_name The part name to which you want to set the text
 * @param[in] text Text you want to set to the part
 */
void view_set_text(Evas_Object *parent, const char *part_name, const char *text)
{
	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	/**
	 * Set text of target part object.
	 */
	elm_object_part_text_set(parent, part_name, text);
}

/**
 * @brief Sets color of the part.
 * @param[in] parent The object has part to which you want to set color
 * @param[in] part_name The name of part to which you want to set color
 * @param[in] r R of RGBA you want to set to the part
 * @param[in] g G of RGBA you want to set to the part
 * @param[in] b B of RGBA you want to set to the part
 * @param[in] a A of RGBA you want to set to the part
 */
void view_set_color(Evas_Object *parent, const char *part_name, int r, int g, int b, int a)
{
	Evas_Object *obj = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	obj = elm_object_part_content_get(parent, part_name);
	if (obj == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get parent.");
		return;
	}

	/**
	 * Set color of target part object.
	 */
	evas_object_color_set(obj, r, g, b, a);
}

/**
 * @brief Creates a naviframe and sets to parent.
 * @param[in] parent The object to which you want to set naviframe object
 */
Evas_Object *view_create_naviframe(Evas_Object *parent)
{
	Evas_Object *nf = NULL;

	/**
	 * Naviframe
	 * Create and initialize elm_naviframe.
	 * Naviframe stands for navigation frame.
	 * elm_naviframe is a views manager for applications.
	 * Naviframe make changing of view is easy and effectively.
	 */

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	nf = elm_naviframe_add(parent);

	elm_object_part_content_set(parent, "elm.swallow.content", nf);

	/**
	 * Add callback function will be operated when back key is pressed.
	 */
	eext_object_event_callback_add(nf, EEXT_CALLBACK_BACK, eext_naviframe_back_cb, NULL);
	eext_object_event_callback_add(nf, EEXT_CALLBACK_MORE, eext_naviframe_more_cb, NULL);

	evas_object_show(nf);

	return nf;
}

/**
 * @brief Pushes item to naviframe object.
 * @param[in] nf Naviframe that item will be added to
 * @param[in] item The object will be added to naviframe
 * @param[in] _pop_cb The function will be operated when this item is popped from naviframe
 * @param[in] cb_data The data passed to the '_pop_cb' function
 */
Elm_Object_Item *view_push_item_to_naviframe(Evas_Object *nf, Evas_Object *item, Elm_Naviframe_Item_Pop_Cb _pop_cb, void *cb_data)
{
	Elm_Object_Item* nf_it = NULL;

	if (nf == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "naviframe is NULL.");
		return NULL;
	}

	if (item == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "item is NULL.");
		return NULL;
	}

	/**
	 * Sixth parameter
	 * @param[in] item_style The name of item style.
	 * "empty" style has no button like prev_button and next_button.
	 */
	nf_it = elm_naviframe_item_push(nf, NULL, NULL, NULL, item, "empty");
	elm_naviframe_item_pop_cb_set(nf_it, _pop_cb, cb_data);

	return nf_it;
}

/**
 * @brief Creates a genlist for circular shape.
 * @param[in] parent Object to which you want to set genlist
 */
Evas_Object *view_create_circle_genlist(Evas_Object *parent)
{
	Evas_Object *genlist = NULL;
	Evas_Object *circle_genlist = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	if (s_info.circle_surface == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "circle surface is NULL.");
		return NULL;
	}

	genlist = elm_genlist_add(parent);
	/**
	 * This make selected list item is shown compressed.
	 */
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	evas_object_smart_callback_add(genlist, "selected", _gl_selected_cb, NULL);

	/**
	 * This make genlist style circular.
	 */
	circle_genlist = eext_circle_object_genlist_add(genlist, s_info.circle_surface);
	eext_circle_object_genlist_scroller_policy_set(circle_genlist, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);
	eext_rotary_object_event_activated_set(circle_genlist, EINA_TRUE);

	evas_object_show(genlist);

	return genlist;
}

/**
 * @brief Adds item to genlist.
 * @param[in] genlist Genlist that item will be added to
 * @param[in] style Style of item determine how to show this item, such as "1text", "1text1icon" and so on
 * @param[in] data Item data that use item's callback function
 * @param[in] _clicked_cb The function will be operated when the item is clicked
 * @param[in] cb_data The data passed to '_clicked_cb' function
 */
Elm_Object_Item *view_append_item_to_genlist(Evas_Object *genlist, const char *style,
		const void *data, Evas_Smart_Cb _clicked_cb, const void *cb_data)
{
	Elm_Genlist_Item_Class *item_class;
	Elm_Object_Item *item;

	if (genlist == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "genlist is NULL.");
		return NULL;
	}

	if (style == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "item style is NULL.");
		return NULL;
	}

	item_class = _set_genlist_item_class(style);
	if (!strcmp(style, "title_with_groupindex") || !strcmp(style, "groupindex")) {
		item = elm_genlist_item_append(genlist, item_class, data, NULL, ELM_GENLIST_ITEM_GROUP, _clicked_cb, cb_data);
	} else {
		item = elm_genlist_item_append(genlist, item_class, data, NULL, ELM_GENLIST_ITEM_NONE, _clicked_cb, cb_data);
	}
	elm_genlist_item_class_free(item_class);

	return item;
}

/**
 * @note
 * Below functions are static.
 */

/*
 * @brief Sets functions will be operated when this item is shown on the screen according to the style.
 * @param[in] style The style of item.
 */
static Elm_Genlist_Item_Class *_set_genlist_item_class(const char *style)
{
	Elm_Genlist_Item_Class *item_class = NULL;

	if (style == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "style is NULL.");
		return NULL;
	}

	item_class = elm_genlist_item_class_new();

	/**
	 * If you want to add the item class of genlist, you should be add below.
	 * To see more genlist's styles click on the link below.
	 * https://developer.tizen.org/development/ui-practices/native-application/efl/ui-components/wearable-ui-components
	 */

	if (!strcmp(style, "title_with_groupindex")) {
		item_class->item_style = "title_with_groupindex";
		/**
		 * This function will be operated when this item is shown on the screen to get the title.
		 */
		item_class->func.text_get = data_get_gl_title_text;
	}	else if (!strcmp(style, "groupindex")) {
		item_class->item_style = "groupindex";
		item_class->func.text_get = data_get_gl_type_text;
	} else if (!strcmp(style, "1text.1icon")) {
		item_class->item_style = "1text.1icon";
		item_class->func.text_get = data_get_gl_text;
		item_class->func.content_get = _get_gl_icon;
	} else if (!strcmp(style, "padding")) {
		/**
		 * "padding" style does nothing.
		 * But it makes genlist's item placed in the middle of the screen.
		 */
	}

	return item_class;
}

/**
 * @brief Function will be operated when window is deleted.
 * @param[in] data The data needed in this function
 * @param[in] obj The smart object
 * @param[in] event_info Information of event
 */
static void _win_delete_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

/**
 * @brief Function will be operated when genlist's item is selected.
 * @param[in] data The data needed in this function
 * @param[in] obj The genlist object
 * @param[in] event_info Selected item
 */
static void _gl_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *it = (Elm_Object_Item *) event_info;

	elm_genlist_item_selected_set(it, EINA_FALSE);
}

/**
 * @brief Function will be operated when genlist's items are shown on the screen.
 * @param[in] data The data passed from 'elm_genlist_item_append' as third parameter
 * @param[in] obj The genlist object
 * @param[in] part Name string of one of the existing text parts in the EDJ group implementing the item's theme
 */
static Evas_Object* _get_gl_icon(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *icon_layout = NULL;

	contact_info_s *contact_info = (contact_info_s *) data;

	if (strcmp(part, "elm.icon")) return NULL;

	icon_layout = _create_icon_layout(obj, contact_info->name, contact_info->img_path, NULL);

	return icon_layout;

}

/**
 * @brief Creates icon layout that is located in genlist's item.
 * @param[in] parent The object to which you want to add this layout
 * @param[in] name Text you want to set to icon layout
 * @param[in] img_path Path of image file you want to set to icon layout
 * @param[in] user_data The data needed in this function
 */
Evas_Object *_create_icon_layout(Evas_Object *parent, const char *name, const char *img_path, void *user_data)
{
	Evas_Object *icon_layout = NULL;
	char edj_path[PATH_MAX] = { 0, };

	data_get_resource_path(EDJ_FILE, edj_path, sizeof(edj_path));

	icon_layout = view_create_layout(parent, edj_path, "list-icon", NULL, user_data);

	if (img_path == NULL) {
		/**
		 *  Set default image that contains name's initial if the contact has not image file.
		 */
		char default_img_path[PATH_MAX] = { 0, };
		char buf[128] = { 0, };
		char first_char = name[0];

		data_get_resource_path(IMAGE_CONTACT_BG, default_img_path, sizeof(default_img_path));

		view_set_image(icon_layout, "bg.swallow", default_img_path);
		view_set_color(icon_layout, "bg.swallow", 207, 115, 58, 255);

		snprintf(buf, sizeof(buf), "%c", first_char);
		view_set_text(icon_layout, "txt.first.char", buf);
	} else {
		/**
		 * Set specific image if the contact has image file.
		 */
		view_set_image(icon_layout, "bg.swallow", img_path);
	}

	return icon_layout;
}

/* End of file */
